from .mg import OptionHelper

class Layout(OptionHelper):
    _allowed_layout = [
        "buffer",
        "bottom",
        "height",
        "left",
        "right",
        "small_height_threshold",
        "small_width_threshold",
        "top",
        "width"
    ]

    def set_margin(self, top=40, bottom=30, left=50, right=10, buffer_size=8):
        self.set_integer("top", top)
        self.set_integer("bottom", bottom)
        self.set_integer("left", left)
        self.set_integer("right", right)
        self.set_integer("buffer", buffer_size)

    def set_size(self, height=220, width=350,
                 height_threshold=120,
                 width_threshold=160):
        self.set_integer("height", height)
        self.set_integer("width", width)
        self.set_integer("small_height_threshold", height_threshold)
        self.set_integer("small_width_threshold", width_threshold)

    def get(self):
        return {k:v for k,v in list(self.options.items()) if k in self._allowed_layout}


from .react_template import ReactComponent

class UIComponent(object):
    """
    """
    def __init__(self, component_type, options, route_func):
        """"""
        self.params = {
            "type": component_type,
            "options": options
        }
        self.route_func = route_func

    def register_route(self, app):
        """"""
        if "url" not in self.params["options"]:
            raise Exception("Component does not have a URL property")

        if not hasattr(self.route_func, "__call__"):
            raise Exception("No app route function supplied")

        app.add_url_rule(self.params["options"]["url"],
            self.params["options"]["url"],
            self.route_func)
        app.view_functions[self.params["options"]["url"]] = self.route_func

class SimpleComponent(object):
    def __init__(self, layout, src_file, component_id, props):
        self.layout = layout
        self.src_file = src_file
        self.component_id = component_id
        self.props = props

    def render(self, path):
        return ReactComponent(
            self.layout,
            self.src_file,
            self.component_id,
            props=self.props,
            static_path=path)

class UILayout(object):

    def __init__(self, layout, src_file, component_id, dynamic=True, filter_style="'btn-group'"):
        self.layout = layout
        self.src_file = src_file
        self.component_id = component_id
        self.filter_style = filter_style
        self.filters = []
        self.charts = []
        self.dynamic = dynamic

    def add_filter(self, component):
        if getattr(component, "name") != "Filter":
            raise Exception("Component is not an instance of Filter")
        self.filters.append(component)

    def add_chart(self, component):
        if getattr(component, "name") != "Chart":
            raise Exception("Component is not an instance of Chart")
        self.charts.append(component)

    def build_props(self):
        props = {}
        if self.filters:
            props["filters"] = [f.params for f in self.filters]
        if self.charts:
            props["charts"] = [c.params for c in self.charts]

        props["dynamic"] = self.dynamic
        props["filter_style"] = self.filter_style
        return props

    def assign_routes(self, app):
        for f in self.filters:
            if f.route_func:
                f.register_route(app)

        for c in self.charts:
            if c.route_func:
                c.register_route(app)

    def render_layout(self, app, path):
        self.assign_routes(app)
        return ReactComponent(
            self.layout,
            self.src_file,
            self.component_id,
            props=self.build_props(),
            static_path=path)



from react import jsx
from jinja2 import Template
import json

class ReactTemplate(object):
    """
    """
    def __init__(self, template, template_args, path):
        self.transformer = jsx.JSXTransformer()
        self.template = template
        self.args = template_args
        self.path = path

    def write_to_file(self, s):
        """
        """
        f = open(self.path, 'w')
        f.write(s)
        f.close()

    def to_js(self):
        """
        """
        t = Template(self.template)
        js = self.transformer.transform_string(t.render(**self.args))
        self.write_to_file(js)

def format_props(props):
    """
    """
    vars_ = []
    props_ = []
    for k, v in list(props.items()):
        vars_.append(Template("var {{k}} = {{v}};").render(k=k,v=json.dumps(v)))
        props_.append(Template("{{k}} = {{v}}").render(k=k, v="{"+k+"}"))
    return "\n".join(vars_), "\n".join(props_)


class ReactComponent(ReactTemplate):
    _react = (
    """
    var Component = require("{{path}}").{{name}};
    {{vars}}
    React.render(
        <Component
        {{props}} />,
        document.getElementById("{{id}}")
    );
    """)
    def __init__(self, name, path, element_id, props={}, static_path=""):
        vars_, props_ = format_props(props)

        params = {
            "name": name,
            "path": path,
            "vars": vars_,
            "props": props_,
            "id": element_id
        }
        for k, v in list(props.items()):
            params[k] = v

        super(ReactComponent, self).__init__(
            self._react, params, static_path)
        self.to_js()





from ..ui import UIComponent
import pandas as pd
from flask import request, jsonify, make_response

class Chart(UIComponent):
    name = "Chart"

    @staticmethod
    def apply_filters(df, filters):
        idx = pd.Series([True]*df.shape[0])
        for k, v in list(filters.items()):
            if k not in df.columns:
                continue
            idx &= (df[k] == v)

        return df.loc[idx]

class LinePlot(Chart):
    def __init__(self, chart_id, url, plot_object):
        opts = {
            "url": url,
            "chartid": chart_id
        }
        super(LinePlot, self).__init__("LinePlot", opts, plot_object.api_route)




from .mg import OptionHelper

class Axes(OptionHelper):
    _allowed_axes = [
        "inflator",
        "max_x", "max_y",
        "min_x", "min_y",
        "min_y_from_data",
        "missing_text",
        "show_missing_background",
        "show_year_markers",
        "show_secondary_x_label",
        "small_text",
        "xax_count",
        "xax_format",
        "x_axis",
        "x_extended_ticks",
        "x_label",
        "xax_start_at_min",
        "xax_units",
        "xax_tick_length",
        "y_axis",
        "y_extended_ticks",
        "y_label",
        "y_scale_type",
        "yax_count",
        "yax_format",
        "yax_tick_length",
        "yax_units"
        ]

    def set_inflator(self, value):
        self.set_float("inflator", value)

    def set_xlim(self, xlim):
        if len(xlim) != 2:
            raise ValueError("xlim must contain two elements")

        if xlim[1] < xlim[0]:
            raise ValueError("Min must be less than Max")

        self.options["min_x"] = xlim[0]
        self.options["max_x"] = xlim[1]

    def set_ylim(self, ylim):
        if len(ylim) != 2:
            raise ValueError("ylim must contain two elements")

        if ylim[1] < ylim[0]:
            raise ValueError("Min must be less than Max")

        self.options["min_y"] = ylim[0]
        self.options["max_y"] = ylim[1]

    def set_min_y_from_data(self, value):
        self.set_boolean("min_y_from_data", value)

    def show_year_markers(self, value):
        self.set_boolean("show_year_markers", value)

    def show_secondary_x_label(self, value):
        self.set_boolean("show_secondary_x_label", value)

    def set_small_text(self, value):
        self.set_boolean("small_text", value)

    def show_x_extended_ticks(self, value):
        self.set_boolean("x_extended_ticks", value)

    def show_y_extended_ticks(self, value):
        self.set_boolean("y_extended_ticks", value)

    def show_xaxis(self, value):
        self.set_boolean("x_axis", value)

    def show_yaxis(self, value):
        self.set_boolean("y_axis", value)

    def set_xlabel(self, label):
        self.set_string("x_label", label)

    def set_ylabel(self, label):
        self.set_string("x_label", label)

    def set_xticks_count(self, value):
        self.options["xax_count"] = value

    def set_yticks_count(self, value):
        self.options["yax_count"] = value

    def xaxis_start_at_min(self, value):
        self.set_boolean("xax_start_at_min", value)

    def set_xticks_length(self, value):
        self.set_integer("xax_tick_length", value)

    def set_yticks_length(self, value):
        self.set_integer("yax_tick_length", value)

    def set_xunits(self, value):
        self.set_string("xax_units", value)

    def set_yunits(self, value):
        self.set_string("yax_units", value)

    def logscale(self):
        self.set_boolean("y_scale_type", True)

    def set_xformat(self, value):
        self.set_string("xax_format", value)

    def set_yformat(self, value):
        self.set_string("yax_format", value)

    def get(self):
        return {k:v for k,v in list(self.options.items()) if k in self._allowed_axes}




from . import filters

Filter = filters.Filter
DownloadButton = filters.DownloadButton
ConditionalButton = filters.ConditionalButton
SliderInput = filters.SliderInput
SelectButton = filters.SelectButton
ApiButton = filters.ApiButton
DynamicTextInput = filters.DynamicTextInput




from .mg import MG
from flask import jsonify, request

class LineChart(MG):
    def __init__(self, df, figure, x, y, title="Line Chart",
        description="Line Chart", init_params={}, timeseries=False):

        self.x = x
        self.y = y
        self.plot_opts = {
            "title": title,
            "description": description,
            "target": "#"+figure.chart_id,
            "x_accessor": "x",
            "y_accessor": "y",
            "init_params": init_params
        }
        self.timeseries = timeseries
        for k, v in list(figure.get().items()):
            self.plot_opts[k] = v

        def get_data():
            args = {}
            for c in init_params:
                if request.args.get(c):
                    args[c] = request.args[c]
                else:
                    args[c] = init_params[c]
            return jsonify(self.to_json(
                    self.apply_filters(df, args)
                ))

        super(LineChart, self).__init__(figure.chart_id, figure.url, self.plot_opts, get_data)

    def to_json(self, df):
        values = [[]]*len(self.y)
        values = {k: [] for k in self.y}
        for i, row in df.iterrows():
            for yy in self.y:
                values[yy].append({
                    "x": row[self.x],
                    "y": row[yy]
                    })
        return {"result":  [values[k] for k in self.y], "date": self.timeseries}




from .nvd3 import NVD3
from flask import jsonify, request
import numpy as np

class TwoAxisFocus(NVD3):
    _allowed_axes = ["sigma", "minmax"]
    def __init__(self, x, y1, y2, data_source, init_params={},
        chart_id="new_chart", url="/new_chart/", colors=[], auto_scale="sigma",
        y1_axis_range=[], y2_axis_range=[], sigma=3,
        x_label="", y1_label="", y2_label="",
        margin={"top": 30, "right": 60, "bottom": 50, "left": 70}):

        self.x = x
        self.y1 = y1
        self.y2 = y2
        self.auto_scale = auto_scale if auto_scale in self._allowed_axes else "sigma"
        self.sigma = 3
        self.y1_axis_range = y1_axis_range
        self.y2_axis_range = y2_axis_range

        self.options = {
            "type": "TwoAxisFocus",
            "chartid": chart_id,
            "url": url,
            "colors": colors,
            "init_params": init_params,
            "labels": {
                "xAxis": x_label,
                "yAxis1": y1_label,
                "yAxis2": y2_label
            },
            "margin": margin,
            "type": "TwoAxisFocus"
        }
        def get_data():
            args = {}
            for c in init_params:
                if request.args.get(c):
                    args[c] = request.args[c]
                else:
                    args[c] = init_params[c]
            return jsonify(self.to_json(
                    self.apply_filters(data_source, args)
                ))

        super(TwoAxisFocus, self).__init__(self.options, get_data)

    def get_bounds(self, y, method="sigma"):
        if self.auto_scale == "sigma":
            m_, s_ = y.mean(), y.std()
            l = m_ - self.sigma*s_
            u = m_ + self.sigma*s_
        else:
            l = y.min()
            u = y.max()
        return [l, u]

    def to_json(self, df):
        if df.empty:
            return {
                "data": [],
                "yAxis1": {"lower": 0, "upper": 1},
                "yAxis2": {"lower": 0, "upper": 1}
            }

        if not self.y1_axis_range:
            bounds1 = self.get_bounds(df[self.y1], method=self.auto_scale)
        else:
            bounds1 = self.y1_axis_range

        if not self.y2_axis_range:
            bounds2 = self.get_bounds(df[self.y2], method=self.auto_scale)
        else:
            bounds2 = self.y2_axis_range

        records = [
            {"key": self.y1, "values": [], "yAxis": 1, "type": "line"},
            {"key": self.y2, "values": [], "yAxis": 2, "type": "line"}
        ]

        for n, r in df.iterrows():
            records[0]["values"].append({"x": r[self.x], "y": r[self.y1]})
            records[1]["values"].append({"x": r[self.x], "y": r[self.y2]})

        return {
            "data": records,
            "yAxis1": {"bounds": bounds1},
            "yAxis2": {"bounds": bounds2}
            }



from ..charts import Chart

class MG(Chart):
    def __init__(self, chart_id, url, params, api_route):
        opts = {
            "url": url,
            "chart_id": chart_id,
            "params": params
        }
        super(MG, self).__init__("MetricsGraphics", opts, api_route)

class OptionHelper(object):
    def __init__(self):
        self.options = {}

    def set_float(self, option, value):
        if not isinstance(value, float):
            raise TypeError("Inflator must be a float")
        self.options[option] = value

    def set_integer(self, option, value):
        try:
            int_value = int(value)
        except ValueError as err:
            print(err.args)

        self.options[option] = value

    def set_boolean(self, option, value):
        if not isinstance(value, bool):
            raise TypeError("%s must be a boolean" % option)

        self.options[option] = str(value).lower()

    def set_string(self, option, value):
        if not isinstance(value, str):
            raise TypeError("%s must be a string" % option)

        self.options[option] = value





from .mg import OptionHelper

class Graphic(OptionHelper):
    _allowed_graphics = [
        "aggregate_rollover",
        "animate_on_load",
        "area",
        "baselines",
        "chart_type",
        "custom_line_color_map",
        "decimals",
        "error",
        "format",
        "full_height",
        "full_width",
        "interpolate",
        "interpolate_tension",
        "legend",
        "legend_target",
        "linked",
        "lined_format",
        "list",
        "markers",
        "max_data_size",
        "mouseover",
        "mousemove",
        "mouseout",
        "point_size",
        "show_confidence_band",
        "show_rollover_text",
        "show_tooltips",
        "target",
        "transition_on_update",
        "x_rug",
        "y_rug"
    ]

    _allowed_charts = ["line", "histogram", "point", "missing-data"]

    _allowed_interpolates = [
        "cardinal",
        "linear",
        "linear-closed",
        "step",
        "step-before",
        "step-after",
        "basis",
        "basis-open",
        "basis-closed",
        "bundle",
        "cardinal-open",
        "cardinal-closed",
        "monotone"
    ]

    def aggregate_rollover(self):
        self.set_boolean("aggregate_rollover", True)

    def animate_on_load(self):
        self.set_boolean("animate_on_load", True)

    def area(self, value=True):
        self.set_boolean("area", value)

    def add_baseline(self, value, label="baseline"):
        self.options["baselines"] = [{"value": value, "label": label}]

    def chart_type(self, value):
        if value not in self._allowed_charts:
            raise ValueError("Not a valid chart type")

        self.options["chart_type"] = value

    def custom_line_color_map(self, values):
        if not isinstance(values, list):
            raise TypeError("custom_line_color_map must be a list")

        self.options["custom_line_color_map"] = values

    def decimals(self, value):
        self.set_integer("decimals", value)

    def error(self, value):
        self.set_string("error", value)

    def format(self, value):
        if value not in ["count", "percentage"]:
            raise ValueError("Not a valid format. Must be count or percentage")

        self.options["format"] = value

    def full_height(self):
        self.set_boolean("full_height", True)

    def full_width(self):
        self.set_boolean("full_width", True)

    def interpolate(self, value):
        if value not in _allowed_interpolates:
            raise ValueError("Not a valid interpolation method")

        self.options["interpolate"] = value

    def interpolate_tension(self, value):
        if (value < 0) or (value > 1):
            raise ValueError("Tension must in [0, 1]")

        self.options["interpolate_tension"] = value

    def legend(self, values):
        if not isinstance(values, list):
            raise TypeError("legend must be a list of labels")

        self.options["legend"] = values

    def legend_target(self, value):
        self.set_string("legend_target", value)

    def linked(self):
        self.set_boolean("linked", True)

    def linked_format(self, value):
        self.set_string("linked_format", value)

    def as_list(self):
        self.set_boolean("list", True)

    def markers(self, values):
        if not isinstance(values, list):
            raise TypeError("Markers must be a list of objects")

        self.options["markers"] = values

    def max_data_size(self, value):
        self.set_integer("max_data_size", value)

    def missing_text(self, value):
        self.set_string("missing_text", value)

    def show_missing_background(self, value=True):
        self.set_boolean("show_missing_background", value)

    def point_size(self, value):
        self.set_float("point_size", value)

    def show_confidence_band(self, value):
        if not isinstance(values, list):
            raise TypeError("show_confidence_band must be a list of strings")

        self.options["show_confidence_band"] = values

    def show_rollover_text(self, value=True):
        self.set_boolean("show_rollover_text", value)

    def show_tooltips(self, value=True):
        self.set_boolean("show_tooltips", value)

    def target(self, value):
        self.set_string("target", value)

    def transition_on_update(self, value=True):
        self.set_boolean("transition_on_update", value)

    def x_rug(self):
        self.set_boolean("x_rug", True)

    def y_rug(self):
        self.set_boolean("y_rug", True)

    def get(self):
        return {k:v for k,v in list(self.options.items()) if k in self._allowed_graphics}



from .nvd3 import NVD3
from flask import jsonify, request
import numpy as np

class PieChart(NVD3):
    def __init__(self, values, df, init_params={},
        chart_id="piechart", url="/piechart/", colors=[]):

        self.options = {
            "type": "PieChart",
            "chartid": chart_id,
            "url": url,
            "colors": colors
        }

        self.values = values
        def get_data():
            args = {}
            for c in init_params:
                if request.args.get(c):
                    args[c] = request.args[c]
                else:
                    args[c] = init_params[c]
            return jsonify(self.to_json(
                        self.apply_filters(df, args)
                    ))

        super(PieChart, self).__init__(self.options, get_data)

    def to_json(self, df):

        records = []
        sum_ = np.sum([df[c].iloc[0] for c in self.values])
        for c in self.values:
            records.append({
                "label": self.values[c],
                "value": "%.2f"%np.around(df[c].iloc[0] / sum_, decimals=2)
                })
        return {
            "data" : records
        }



from ..charts import Chart
import pandas as pd
from flask import request, jsonify, make_response

_COLOR_MAP = {
    'light blue':'#add8e6',
    "antique gold":'#fff4b0',
    "antique silver":'#d7cdc4',
    "beige": '#f5f5dc',
    "black":'#000000',
    "blue": '#8084ff',
    "bronze": '#c95a0b',
    "brown": '#864',
    "burgundy": '#ff7272',
    "burnt orange": '#cc5500',
    "camel": '#c96',
    "canary yellow": '#ffef00',
    "cobalt": "#56b3ff",
    "coral": "#ff9e80",
    "dark green": '#006400',
    "dark grey": '#666666',
    "dark pink": '#e3489b',
    "dark purple": '#540061',
    "fuchsia": '#ff00ff',
    "gold": '#fc0',
    "gray": '#9c9c9c',
    "green": "#83ff7f",
    "grey": "#9c9c9c",
    "jewel tone purple": '#ae2cc6',
    "light green": '#90ee90',
    "light grey": '#d3d3d3',
    "light pink": '#ffd6d3',
    "light purple": '#b0c4de',
    "magenta":  '#ff00ff',
    "mustard":  '#ffe761',
    "navy": '#6c70ff',
    "off-white": '#ffffdd',
    "olive": '#808000',
    "orange": '#ffc870',
    "orange red": '#ff4500',
    "pale yellow": '#ffff9d',
    "pink": '#ffb6c1',
    "purple": '#800080',
    "red": '#ff0000',
    "rose gold": '#ffba9d',
    "silver": '#c0c0c0',
    "soft orange": '#ffc63c',
    "tan":  '#d2b48c',
    "teal": '#008080',
    "teal green":'#a1dfc6',
    "turquoise": '#40e0d0',
    "white": '#ffffff',
    "yellow": '#ffff00',
    "other": '#111111'
}

class Datamap(Chart):
    def __init__(self, chart_id, url, params, api_route):
        opts = {
            "url": url,
            "chartid": chart_id,
            "params": params
        }
        super(Datamap, self).__init__("Datamaps", opts, api_route)

class DatamapUSA(Datamap):
    def __init__(self, url, chart_id, df,
                state_index, color_index,
                init_params={},
                color_map=_COLOR_MAP):

        self.state_index = state_index
        self.color_index = color_index
        self.fills = color_map
        self.fills["defaultFills"] = "black"

        def get_data():
            args = {}
            for c in init_params:
                if request.args.get(c):
                    args[c] = request.args[c]
                else:
                    args[c] = init_params[c]
            return jsonify(self.to_json(
                    self.apply_filters(df, args)
                ))
        super(DatamapUSA, self).__init__(chart_id, url, init_params, get_data)

    def to_json(self, df):
        records = {}
        for i, row in df.iterrows():

            records[row[self.state_index]] = {
                "fillKey": row[self.color_index]
            }

        return {
            "data": records,
            "fills": self.fills
        }



from setuptools import setup

setup(
    name='pyxley',
    version='0.0.6',
    author='Nicholas Kridler',
    author_email='nmkridler@gmail.com',
    license='MIT',
    description='Python tools for building Flask-based web applications',
    packages = ['pyxley', 'pyxley.filters', 'pyxley.charts', 'pyxley.charts.mg',
        'pyxley.charts.datatables', 'pyxley.charts.datamaps',
        'pyxley.charts.nvd3'],
    long_description='Python tools for building Flask-based web applications using React.js',
    url='https://github.com/stitchfix/pyxley',
    keywords=['pyreact', 'flask'],
    classifiers=[
        'Intended Audience :: Developers',
    ],
    install_requires=[
        'pyreact',
        'flask',
        'pandas'
    ]
)



from . import line_chart
from . import scatter_plot
from . import axes
from . import figure
from . import layout
from . import histogram

Axes = axes.Axes
Figure = figure.Figure
Graphics = graphic.Graphic
LineChart = line_chart.LineChart
ScatterPlot = scatter_plot.ScatterPlot
Histogram = histogram.Histogram


from ..ui import UIComponent

class Filter(UIComponent):
    name = "Filter"

class DownloadButton(Filter):
    """
    """
    def __init__(self, label, url, route_func):
        opts = {
            "url": url,
            "label": label
        }
        super(DownloadButton, self).__init__(
            "DownloadButton",
            opts,
            route_func)

class ConditionalButton(Filter):
    """
    """
    def __init__(self, labels, items, filter_ids, defaults,
                 filter_style="btn-group"):

        if (len(labels) != 2) or (len(defaults) != 2) or (len(filter_ids) != 2):
            raise Exception("The length of labels, defaults, or filter_ids must be 2")

        item_data = []
        for k, v in list(items.items()):
            item_data.append({"primary": k, "secondary": v})

        opts = {
            "labels": labels,
            "defaults": defaults,
            "aliases": filter_ids,
            "items": item_data,
            "filter_style": filter_style
        }
        super(ConditionalButton, self).__init__("ConditionalSelectButton", opts, None)

class SliderInput(Filter):
    """
    """
    def __init__(self, label, min_val, max_val, filter_id, default, step=1):
        opts = {
            "label": label,
            "min": min_val,
            "max": max_val,
            "step": step,
            "alias": filter_id,
            "default": default

        }
        super(SliderInput, self).__init__("SliderInput", opts, None)

class SelectButton(Filter):
    """
    """
    def __init__(self, label, items, filter_id, default):
        opts = {
            "label": label,
            "items": items,
            "alias": filter_id,
            "default": default
        }
        super(SelectButton, self).__init__("SelectButton", opts, None)

class ApiButton(Filter):
    """
    """
    def __init__(self, label, url, filter_id, default, route_func):
        opts = {
            "label": label,
            "url": url,
            "alias": filter_id,
            "default": default
        }
        super(ApiButton, self).__init__(
            "ApiButton",
            opts,
            route_func)

class DynamicTextInput(Filter) :
    """
    """
    def __init__(self, url, filter_id, default, route_func,
            placeholder="Enter text...", help=""):
        opts = {
            "url": url,
            "help": help,
            "alias": filter_id,
            "default": default,
            "placeholder": placeholder,
            "max": 20
        }
        super(DynamicTextInput, self).__init__(
            "DynamicSearch",
            opts,
            route_func)




from . import ui
from . import react_template

ReactTemplate = react_template.ReactTemplate
ReactComponent = react_template.ReactComponent
UIComponent = ui.UIComponent
SimpleComponent = ui.SimpleComponent
UILayout = ui.UILayout





from .mg import MG
from flask import jsonify, request

class Histogram(MG):
    def __init__(self, df, figure, column, bins, title="Histogram",
        description="Histogram", init_params={}):
        self.column = column
        self.plot_opts = {
            "title": title,
            "description": description,
            "init_params": init_params,
            "bins": bins
        }
        figure.graphics.chart_type("histogram")
        figure.graphics.target("#"+figure.chart_id)
        for k, v in list(figure.get().items()):
            self.plot_opts[k] = v

        def get_data():
            args = {}
            for c in init_params:
                if request.args.get(c):
                    args[c] = request.args[c]
                else:
                    args[c] = init_params[c]
            return jsonify(self.to_json(
                    self.apply_filters(df, args)
                ))

        super(Histogram, self).__init__(figure.chart_id, figure.url,
            self.plot_opts, get_data)

    def to_json(self, df):
        return {"result": df[self.column].tolist()}




from .axes import Axes
from .layout import Layout
from .graphic import Graphic

class Figure(object):
    def __init__(self, url, chart_id):
        self.url = url
        self.chart_id = chart_id
        self.axes = Axes()
        self.graphics = Graphic()
        self.layout = Layout()

    def get(self):
        options = {}
        for x in [self.axes, self.graphics, self.layout]:
            for k, v in list(x.get().items()):
                options[k] = v

        return options




from ..charts import Chart
from flask import jsonify, request

"""
    Wrapper for jquery Datatables
"""
class DataTable(Chart):
    def __init__(self, table_id, url, df, columns={}, init_params={},
        paging=False, searching=False, sortable=False, classname="display", **kwargs):

        opts = {
            "params": init_params,
            "id": table_id,
            "url": url,
            "className": classname,
            "table_options": {
                "paging": paging,
                "searching": searching,
                "bSort": sortable
            }
        }

        for k, v in list(kwargs.items()):
            opts["table_options"][k] = v

        self.columns = columns
        self.confidence = {}
        for k, v in list(self.columns.items()):
            if "confidence" in v:
                self.confidence[k] = v["confidence"]

        def get_data():
            args = {}
            for c in init_params:
                if request.args.get(c):
                    args[c] = request.args[c]
                else:
                    args[c] = init_params[c]
            return jsonify(self.to_json(
                    self.apply_filters(df, args)
                ))
        super(DataTable, self).__init__("Table", opts, get_data)

    def format_row(self, row, bounds):
        for c in self.columns:
            if c not in row:
                continue

            if "format" in self.columns[c]:
                row[c] = self.columns[c]["format"] % row[c]

            if c in bounds:
                b = bounds[c]
                row[c] = [b["min"],row[b["lower"]], row[b["upper"]], b["max"]]

        return row

    def to_json(self, df):
        records = []

        display_cols = list(self.columns.keys())
        if not display_cols:
            display_cols = list(df.columns)

        bounds = {}
        for c in self.confidence:
            bounds[c] = {
                "min": df[self.confidence[c]["lower"]].min(),
                "max": df[self.confidence[c]["upper"]].max(),
                "lower": self.confidence[c]["lower"],
                "upper": self.confidence[c]["upper"]
            }

        labels = {}
        for c in display_cols:
            if "label" in self.columns[c]:
                labels[c] = self.columns[c]["label"]
            else:
                labels[c] = c

        for i, row in df.iterrows():
            row_ = self.format_row(row, bounds)
            records.append({labels[c]: row_[c] for c in display_cols})

        return {
            "data": records,
            "columns": [{"data": labels[c]} for c in display_cols]
        }
